Package: leiden
Type: Package
Title: R Implementation of Leiden Clustering Algorithm
Version: 0.4.3.1
Date: 2023-11-08
Authors@R: c(person("S. Thomas", "Kelly", email = "tomkellygenetics@gmail.com", role = c("aut", "cre", "trl")),
           person("Vincent A.", "Traag", email = "v.a.traag@cwts.leidenuniv.nl", role = c("com")))
Description: Implements the 'Python leidenalg' module to be called in R.
    Enables clustering using the leiden algorithm for partition a graph into communities.
    See the 'Python' repository for more details: <https://github.com/vtraag/leidenalg>
    Traag et al (2018) From Louvain to Leiden: guaranteeing well-connected communities. <arXiv:1810.08473>.
License: GPL-3 | file LICENSE
URL: https://github.com/TomKellyGenetics/leiden
BugReports: https://github.com/TomKellyGenetics/leiden/issues
Imports: methods, reticulate, Matrix, igraph (>= 1.2.7)
Encoding: UTF-8
RoxygenNote: 7.2.3
Suggests: bipartite, covr, data.table, devtools, graphsim, knitr,
        markdown, multiplex, multinet, network, qpdf, RColorBrewer,
        remotes, rmarkdown, spelling, testthat, tibble
Language: en-US
VignetteBuilder: knitr
Collate: 'find_partition.R' 'leiden.R' 'py_objects.R'
NeedsCompilation: no
Packaged: 2023-11-13 08:25:25 UTC; kelly-t
Author: S. Thomas Kelly [aut, cre, trl],
  Vincent A. Traag [com]
Maintainer: S. Thomas Kelly <tomkellygenetics@gmail.com>
Repository: CRAN
Date/Publication: 2023-11-17 10:20:23 UTC
Built: R 4.6.0; ; 2026-04-27 15:41:47 UTC; unix
