# Generated by roxygen2: do not edit by hand

S3method(leiden,Matrix)
S3method(leiden,data.frame)
S3method(leiden,default)
S3method(leiden,igraph)
S3method(leiden,list)
S3method(leiden,matrix)
export(leiden)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(Matrix,dgeMatrix)
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,as.undirected)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_edgelist)
importFrom(igraph,bipartite_mapping)
importFrom(igraph,cluster_leiden)
importFrom(igraph,communities)
importFrom(igraph,edge_attr)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,is.igraph)
importFrom(igraph,is_bipartite)
importFrom(igraph,is_directed)
importFrom(igraph,is_named)
importFrom(igraph,is_weighted)
importFrom(igraph,laplacian_matrix)
importFrom(igraph,membership)
importFrom(igraph,set_edge_attr)
importFrom(igraph,set_vertex_attr)
importFrom(igraph,simplify)
importFrom(igraph,vertex_attr)
importFrom(igraph,which_loop)
importFrom(igraph,which_mutual)
importFrom(methods,as)
importFrom(methods,is)
importFrom(reticulate,import)
importFrom(reticulate,py_get_attr)
importFrom(reticulate,py_has_attr)
importFrom(reticulate,py_set_attr)
importFrom(reticulate,py_to_r)
importFrom(reticulate,r_to_py)
importFrom(utils,capture.output)
importFrom(utils,install.packages)
